#ifndef AG102_REGS
#define AG102_REGS
/* Check to see if this file has been included already.  */

#ifndef __ASSEMBLER__
#include <nds32_intrinsic.h>
#endif

/* System Clock */
#define MHz                     1000000

#define CPUFREQ                 (495 * MHz)
#define HCLKFREQ                (123 * MHz)
#define PCLKFREQ                (61 * MHz)
#define UCLKFREQ                (33 * MHz)


/* Different memory size defines used by SDRAM initialization */
#define SDRAM_256MB             0x0100
#define SDRAM_128MB             0x00C0
#define SDRAM_64MB              0x0040
#define SDRAM_32MB              0x0004
#define SDRAM_16MB              0x0000
#define ORIG_RAM_BASE           0x80000000
#define REMAP_ROM_BASE          ORIG_RAM_BASE

/************************************
 *  TARGET TIMER CONSTANTS          *
 ************************************/
#define DDR2C_BASE              0x90500000       /* Define base for all ddr2 controller registers */
#define DDR2C_CCR_OFFSET        0x00             /* Controller Configuration Register */
#define DDR2C_DCR_OFFSET        0x04             /* DRAM Configuration Register */
#define DDR2C_IOCR_OFFSET       0x08             /* I/O Configuration Register */
#define DDR2C_CSR_OFFSET        0x0C             /* Controller Status Register */
#define DDR2C_DRR_OFFSET        0x10             /* DRAM Refresh Register */
#define DDR2C_TPR_OFFSET(N)     (0x14 + 4 * (N)) /* SDRAM Timing Parameters Registers 0-2 */
#define DDR2C_GDLLCR_OFFSET     0x20             /* Global DLL Control Register */
#define DDR2C_DLLCR_OFFSET(N)   (0x24 + 4 * (N)) /* DLL Control Registers 0-9 */
#define DDR2C_RSLR_OFFSET(N)    (0x4C + 4 * (N)) /* Rank System Latency Registers 0-3 */
#define DDR2C_RDGR_OFFSET(N)    (0x5C + 4 * (N)) /* Rank DQS Gating Registers 0-3 */
#define DDR2C_DQTR_OFFSET(N)    (0x6C + 4 * (N)) /* DQ Timing Registers 0-8 */
#define DDR2C_DQSTR_OFFSET      0x90             /* DQS Timing Register */
#define DDR2C_DQSBTR_OFFSET     0x94             /* DQS_b Timing Register */
#define DDR2C_ODTCR_OFFSET      0x98             /* ODT Configuration Register */
#define DDR2C_DTR_OFFSET(N)     (0x9C + 4 * (N)) /* Data Training Registers 0-1 */
#define DDR2C_DTAR_OFFSET       0xA4             /* Data Training Address Register */
#define DDR2C_MR_OFFSET         0x1F0            /* Mode Register */

/************************************
 *  TARGET TIMER CONSTANTS          *
 ************************************/
#define TIMER_BASE              0x94900000  /* Define base for all timer registers */
#define TIMER_CNT_OFFSET(N)     (0x00 + 0x10 * (N)) /* Offset for timer N counter register */
#define TIMER_LOAD_OFFSET(N)    (0x04 + 0x10 * (N)) /* Offset for timer N load register */
#define TIMER_MATCH1_OFFSET(N)  (0x08 + 0x10 * (N)) /* Offset for timer N match1 register */
#define TIMER_MATCH2_OFFSET(N)  (0x0C + 0x10 * (N)) /* Offset for timer N match2 register */
#define TIMER_CNTRL_OFFSET      0x30        /* Offset for timer control register */
#define TIMER_INTS_OFFSET       0x34        /* Offset for timer interrupt status register */
#define TIMER_INTM_OFFSET       0x38        /* Offset for timer interrupt mask register */

/* Timer 1 ~ 3, ATC010 */
#define TM_CNTR(N)              (TIMER_BASE + TIMER_CNT_OFFSET(N))
#define TM_LOAD(N)              (TIMER_BASE + TIMER_LOAD_OFFSET(N))
#define TM_MTCH1(N)             (TIMER_BASE + TIMER_MATCH1_OFFSET(N))
#define TM_MTCH2(N)             (TIMER_BASE + TIMER_MATCH2_OFFSET(N))

#define TM_CR                   (TIMER_BASE + TIMER_CNTRL_OFFSET)
#define TM_STA                  (TIMER_BASE + TIMER_INTS_OFFSET)
#define TM_MSK                  (TIMER_BASE + TIMER_INTM_OFFSET)

/************************************
 *  TARGET INTERRUPT CONSTANTS      *
 ************************************/
#define INTC_BASE               0x90F00000  /* Interrupt Controller base address */
#define IRQREV_OFFSET           0x0         /* INTC revision register */
#define IRQOWNER_OFFSET         0x4         /* CPU owner DC regsiter */
#define IRQCPUID0_OFFSET        0x8         /* CPU ID configuration register for interrupt input 0-15 */
#define IRQCPUID1_OFFSET        0xc         /* CPU ID configuration register for interrupt intpu 16-31 */
#define IRQTMR_OFFSET           0x20        /* Interrupt trigger mode register */
#define IRQTLR_OFFSET           0x24        /* Interrupt trigger level register */
#define FIQTMR_OFFSET           IRQTMR_OFFSET
#define FIQTLR_OFFSET           IRQTLR_OFFSET
#define IRQSRC_OFFSET           0x28        /* Interrupt source register */
#define IRQIPI_TRIG_OFFSET      0x40        /* IPI trigger register */
#define IRQIPI_STAT_OFFSET      0x44        /* IPI status / clear register */
#define IRQIPI_PRIO_OFFSET      0x48        /* IPI priority level register */
#define IRQIPIGLB_STAT_OFFSET   0x4c        /* IPI global status / clear register */
#define IRQIPIGLB_PRIO_OFFSET   0x50        /* IPI global priority register */
#define IRQER_OFFSET            0x80        /* Interrupt enable register */
#define IRQICR_OFFSET           0x84        /* Interrupt status / clear register */
#define FIQER_OFFSET            IRQER_OFFSET
#define FIQICR_OFFSET           IRQICR_OFFSET
#define IRQICR_IVIC_OFFSET(N)   (0x88 + 4*(N)) /* IVIC HW0 ~ HW5 interrupt status / clear register */
#define IRQPRI_OFFSET(N)        (0xa0 + 4 *(N))/* Interrupt priority level configuration for interrupt input 0-7 */

#define INT_ER_INIT             0x00000000  /* values used to disable all interrupts */
#define INT_CLEAR_VALUE         0xFFFFFFFF  /* values used to clear all pending interrupts */

/************************************
 *  TARGET DECODER CONSTANTS        *
 ************************************/
#define AHBC_BASE               0x90C00000  /* AHB Controller base address */
#define DEVICE4_OFFSET          0x10        /* Device 4 Control Register offset */
#define DEVICE6_OFFSET          0x18        /* Device 6 Control Register offset */
#define INTC_OFFSET             0x88        /* Interrupt Control Register offset */

#define AHBC_DEV6_INIT          0x10080000  /* AHBC Device 6 init value */
#define AHBC_REMAP_BIT          0x00000001  /* AHBC remap bit for Interrupt Control Register */

/************************************
 * PCU - AG102 Core APB             *
 ************************************/
#define PCU_BASE                0x94800000  /* Device base address */
#define PCU_RR_OFFSET           0x00        /* PCU revision register */
#define PCU_SPIR_OFFSET         0x04        /* Scratch pad info register */
#define PCU_SOCID_OFFSET        0x10        /* SoC ID */
#define PCU_AHBCFG_OFFSET       0x14        /* SoC AHB configuration register */
#define PCU_APBCFG_OFFSET       0x18        /* SoC APB configuration register */
#define PCU_DCSRC0_OFFSET       0x20        /* Driving capability and slew rate control register 0 */
#define PCU_DCSRC1_OFFSET       0x24        /* Driving capability and slew rate control register 1 */
#define PCU_DCSRC2_OFFSET       0x28        /* Driving capability and slew rate control register 2 */
#define PCU_MFPS0_OFFSET        0x30        /* Multi-Function port setting register 0 */
#define PCU_MFPS1_OFFSET        0x34        /* Multi-Function port setting register 1 */
#define PCU_DMASEL_OFFSET       0x38        /* DMA engine selection */
#define PCU_OSCC_OFFSET         0x40        /* OSC control register */
#define PCU_PWMCD_OFFSET        0x44        /* PMW clock divider */
#define PCU_SOC_MISC_OFFSET     0x48        /* SoC misc */
#define PCU_BSMC_OFFSET         0x80        /* BSM control register */
#define PCU_BSMS_OFFSET         0x84        /* BSM status register */
#define PCU_WESENS_OFFSET       0x88        /* Wakeup event sensitivity register */
#define PCU_WESTAT_OFFSET       0x8c        /* Wakeup event status register */
#define PCU_RT_OFFSET           0x90        /* Reset timing register */
#define PCU_ISR_OFFSET          0x94        /* PCU interrupt status register */
#define PCU_PMSP_MEM0_OFFSET(N) (0x400 + 4 * (N)) /* Power Manager Scratch Pad Memory 0 */

/* PCSn registers (n = 1 ~ 9) */
#define PCU_PCSC_OFFSET(N)      (0xa0 + 0x20 * ((N) - 1)) /* PCSn configuration register (clock scaling slot) */
#define PCU_PCSP_OFFSET(N)      (0xa4 + 0x20 * ((N) - 1)) /* PCSn parameter register (clock scaling slot) */
#define PCU_PCSSTAT1_OFFSET(N)  (0xa8 + 0x20 * ((N) - 1)) /* PCSn status register 1 (clock scaling slot) */
#define PCU_PCSSTAT2_OFFSET(N)  (0xac + 0x20 * ((N) - 1)) /* PCSn status register 2 (clock scaling slot) */
#define PCU_PSC1PDD_OFFSET(N)   (0xb0 + 0x20 * ((N) - 1)) /* PCSn PDD register (clock scaling slot) */

/************************************
 *  TARGET GPIO CONSTANTS           *
 ************************************/
#define GPIO_BASE               0x94C00000  /* GPIO base address */
#define GPIO_DOUT_OFFSET        0x0
#define GPIO_DIN_OFFSET         0x4
#define GPIO_DIR_OFFSET         0x8
#define GPIO_DSET_OFFSET        0x10
#define GPIO_DCLEAR_OFFSET      0x14
#define GPIO_PE_OFFSET          0x18
#define GPIO_PT_OFFSET          0x1C
#define GPIO_IE_OFFSET          0x20
#define GPIO_IS_OFFSET          0x24

#define GPIO_IT_OFFSET          0x34
#define GPIO_IC_OFFSET          0x30
#define GPIO_IB_OFFSET          0x38
#define GPIO_IR_OFFSET          0x3c
#define GPIO_BE_OFFSET          0x40        /* BounceEnable */
#define GPIO_BPS_OFFSET         0x44        /* BouncePreScale */

#define GPIO_USED_GCOV          0x02        /* Which GPIO to do GCOV */
#define GPIO_USED_MASK          0x0A        /* Which GPIOs to use */

/************************************
 *  TARGET UART CONSTANTS           *
 ************************************/
#define UART_BASE               0x94200000 //0x99600000
#define UART_RBR_OFFSET         0x00    /* Receiver Buffer Register for Read */
#define UART_THR_OFFSET         0x00    /* Transmitter Holding Register for Write */
#define UART_IER_OFFSET         0x04    /* Interrupt Enable Register */
/* Baud-Rate Divisor Latch (DL, Offset: 0x00, 0x04 when DLAB = 1) */
#define UART_DLL_OFFSET         0x00
#define UART_DLM_OFFSET         0x04
#define UART_IIR_OFFSET         0x08    /* Interrupt Identification Register */
#define UART_FCR_OFFSET         0x08    /* FIFO Control Register */
/* Prescaler Register (PSR, Offset: 0x08 when DLAB =1) */
#define UART_LCR_OFFSET         0x0C    /* Line Control Register */
#define UART_MCR_OFFSET         0x10    /* Modem Control Register */
#define UART_LSR_OFFSET         0x14    /* Line Status Register for Read */
#define UART_TST_OFFSET         0x14    /* Testing Register for Write */
#define UART_MSR_OFFSET         0x18    /* Modem Status Register */
#define UART_SPR_OFFSET         0x1C    /* Scratch Pad Register */
#define UART_MDR_OFFSET         0x20    /* Mode Definition Register */
#define UART_ACR_OFFSET         0x24    /* Auxiliary Control Register */
#define UART_TXLENL_OFFSET      0x28    /* Transmit Frame-Length Register (Low) */
#define UART_TXLENH_OFFSET      0x2C    /* Transmit Frame-Length Register (High)*/
#define UART_MRXLENL_OFFSET     0x30    /* Maximum Receiver Frame-Length (Low) */
#define UART_MRXLENH_OFFSET     0x34    /* Maximum Receiver Frame-Length (High)*/
#define UART_PLR_OFFSET         0x38    /* Preamble Length Register */
#define UART_FMIIR_PIO_OFFSET   0x3C    /* Interrupt Identification Register in FIR Mode (PIO mode) */
#define UART_FMIIR_DMA_OFFSET   0x3C    /* Interrupt Identification Register in FIR Mode (DMA mode) */
#define UART_FMIIER_PIO_OFFSET  0x40    /* Interrupt Identification Enable Register in FIR Mode (PIO mode) */
#define UART_FMIIER_DMA_OFFSET  0x40    /* Interrupt Identification Enable Register in FIR Mode (DMA mode) */
#define UART_STFF_STS_OFFSET    0x44    /* Status FIFO Line Status Register */
#define UART_STFF_RXLENL_OFFSET 0x48    /* Status FIFO Received Frame-Length Register Low */
#define UART_STFF_RXLENH_OFFSET 0x4C    /* Status FIFO Received Frame-Length Register High */
#define UART_FMLSR_OFFSET       0x50    /* Link Status Register in FIR Mode */
#define UART_FMLSIER_OFFSET     0x54    /* Link Status Interrupt Enable Register in FIR Mode */
#define UART_RSR_OFFSET         0x58    /* Resume Register */
#define UART_RXFF_CNTR_OFFSET   0x5C    /* RX FIFO Count Register */
#define UART_LSTFMLENL_OFFSET   0x60    /* Last Frame Length Register Low */
#define UART_LSTFMLENH_OFFSET   0x64    /* Last Frame Length Register High */
#define UART_FEATURE_OFFSET     0x68    /* Feature Register */
#define UART_REVD1_OFFSET       0x6C    /* Revision Register Digit 1 */
#define UART_REVD2_OFFSET       0x70    /* Revision Register Digit 2 */
#define UART_REVD3_OFFSET       0x74    /* Revision Register Digit 3 */

/************************************
 *  IRQ number                      *
 ************************************/
#define HW0_FIQ                 0
#define HW1_IRQ                 1
#define HW6_SW0                 6

/* Interrupt controller IRQ mapping */
#define IRQ_CFC_CD              0
#define IRQ_CFC_DMA             1
#define IRQ_SSP1                2
#define IRQ_I2C                 3
#define IRQ_SDC                 5
#define IRQ_SSP2                6
#define IRQ_STUART              7
#define IRQ_PMU                 8
#define IRQ_SWI                 9
#define IRQ_FTUART              10
#define IRQ_BTUART              11
#define IRQ_EXTINT5             12
#define IRQ_GPIO                13
#define IRQ_TIMER2              14
#define IRQ_TIMER3              15
#define IRQ_WDT                 16
#define IRQ_RTCALARM            17
#define IRQ_RTCSECOND           18
#define IRQ_TIMER1              19
#define IRQ_CPE_AHB_DMA         21
#define IRQ_CPE_APB_DMA         24
#define IRQ_MAC                 25
#define IRQ_USBDEV              26   // USB device
#define IRQ_EXTINT4             27
#define IRQ_EXTINT0             28
#define IRQ_EXTINT1             29
#define IRQ_EXTINT2             30
#define IRQ_EXTINT3             31

/* interrupt sources */
#define IC_SWI                  (1 << IRQ_SWI)
#define IC_GPIO                 (1 << IRQ_GPIO)
#define IC_TIMER1               (1 << IRQ_TIMER1)
#define IC_TIMER2               (1 << IRQ_TIMER2)
#define IC_TIMER3               (1 << IRQ_TIMER3)


/* Low-level port I/O */
#define inw(reg)        (*((volatile unsigned int *) (reg)))
#define outw(reg, data) ((*((volatile unsigned int *)(reg)))=(unsigned int)(data))

#ifndef NDS32_INTR_DEF
/*
 * NDS32_REG_SET_BITS(addr, mask)
 * Do set bits to 1 at specified location
 * Operation: *addr = (*addr | mask)
 */
#define NDS32_SET_BITS(addr, mask) outw(addr, inw(addr) | mask)

/*
 * NDS32_REG_WAIT4_BIT_ON(addr, mask)
 * Read the contents at addr and use mask to
 * mask off unused bits then wait until all of
 * the remaining bits are on
 *
 * Operation: while(!(*addr & mask));
 */
#define NDS32_REG_WAIT4_BIT_ON(addr, mask) while(0 == (inw(addr) & mask))

/*
 * NDS32_REG_WAIT4_BIT_OFF(addr, mask)
 * Read the contents at addr and use mask to
 * mask off unused bits then wait until all of
 * the remaining bits are off
 *
 * Operation: while((*addr & mask));
 */
#define NDS32_REG_WAIT4_BIT_OFF(addr, mask) while(inw(addr) & mask)

/*
 * NDS32_REG_WRITE(addr, data)
 * Write data to specified location at addr
 *
 * Operation: *addr = data
 * */
#define NDS32_REG_WRITE(addr, data) outw(addr, data)
#define NDS32_REG_READ(addr)        inw(addr)

/*
 * NDS32_FMEMCPY_BYTE(dst, src, size)
 * Do forward (low address to high address) memory copy in byte
 * */
#define NDS32_FMEMCPY_BYTE(dst, src, size) \
	__asm__ __volatile__( \
		"sethi   $r3, hi20(%2)       \n\t" \
		"ori     $r3, $r3, lo12(%2)  \n\t" \
		"sethi   $r1, hi20(%0)       \n\t" \
		"ori     $r1, $r1, lo12(%0)  \n\t" \
		"sethi   $r2, hi20(%1)       \n\t" \
		"ori     $r2, $r2, lo12(%1)  \n\t" \
		"1:\n\t" \
		"lbi.bi  $r0, [$r2], 1       \n\t" \
		"sbi.bi  $r0, [$r1], 1       \n\t" \
		"addi    $r3, $r3, -1        \n\t" \
		"bnez    $r3, 1b             \n\t" \
		::"i"(dst),"i"(src),"i"(size):"$r0", "$r1","$r2","$r3")

#define NDS32_FMEMCPY(dst, src, size) \
	__asm__ __volatile__( \
		"sethi   $r3, hi20(%2)       \n\t" \
		"ori     $r3, $r3, lo12(%2)  \n\t" \
		"sethi   $r1, hi20(%0)       \n\t" \
		"ori     $r1, $r1, lo12(%0)  \n\t" \
		"sethi   $r2, hi20(%1)       \n\t" \
		"ori     $r2, $r2, lo12(%1)  \n\t" \
		"1:\n\t" \
		"lwi.bi  $r0, [$r2], 4       \n\t" \
		"swi.bi  $r0, [$r1], 4       \n\t" \
		"addi    $r3, $r3, -4        \n\t" \
		"bgtz    $r3, 1b             \n\t" \
		::"i"(dst),"i"(src),"i"(size):"$r0", "$r1","$r2","$r3")

#endif

/************************************
 *  HAL Level : Memory interface    *
 ************************************/
#define AG102_DDR_INIT()                                            \
do {                                                                \
        /* SDRAM configure */                                       \
        NDS32_REG_WRITE(DDR2C_BASE + DDR2C_DCR_OFFSET, 0x000025cc); \
        /* Data training addr */                                    \
        NDS32_REG_WRITE(DDR2C_BASE + DDR2C_DTAR_OFFSET, 0x00100000);\
        /* Set refresh / mode / IOCR */                             \
        NDS32_REG_WRITE(DDR2C_BASE + DDR2C_DRR_OFFSET, 0x00034812); \
        NDS32_REG_WRITE(DDR2C_BASE + DDR2C_MR_OFFSET, 0x00000852);  \
        NDS32_REG_WRITE(DDR2C_BASE + DDR2C_IOCR_OFFSET, 0x0000000f);\
        /* Enable host ports and trigger initialization */          \
        NDS32_REG_WRITE(DDR2C_BASE + DDR2C_CCR_OFFSET, 0x80020000); \
        /* Wait for ddr init state to be set */                     \
        __nds32__isb();                                             \
        /* Wait until the config initialization is finish */        \
        while(NDS32_REG_READ(DDR2C_BASE + DDR2C_CSR_OFFSET) >> 23); \
        /* Enable host ports and trigger the data training */       \
        NDS32_REG_WRITE(DDR2C_BASE + DDR2C_CCR_OFFSET, 0x40020004); \
        /* Wait for ddr init state to be set */                     \
        __nds32__isb();                                             \
        /* Wait until the ddr data trainning is complete */         \
        while(NDS32_REG_READ(DDR2C_BASE + DDR2C_CSR_OFFSET) >> 23); \
        register unsigned int v;                                    \
        v = NDS32_REG_READ(DDR2C_BASE + DDR2C_CSR_OFFSET);          \
        NDS32_REG_WRITE(DDR2C_BASE + DDR2C_CSR_OFFSET, 0x00ffffff); \
        if(!(v >> 20)) break;                                       \
} while(1)

#define HAL_MEMORY_SETUP(_base_)                                    \
do {                                                                \
        /* Initial DDR controller */                                \
        AG102_DDR_INIT();                                           \
        /* The RAM is mapped to ORIG_RAM_BASE, for use. */          \
        /* ROM (base,size)=(0M,256M), RAM (base,size)=(2G,256M) */  \
        NDS32_REG_WRITE(AHBC_BASE + DEVICE4_OFFSET, 0x00000000 | 0x80000);   \
        NDS32_REG_WRITE(AHBC_BASE + DEVICE6_OFFSET, ORIG_RAM_BASE | 0x80000);\
} while(0)

#define HAL_MEMORY_REMAP()	NDS32_SET_BITS(AHBC_BASE + INTC_OFFSET, 0x1)

#define HAL_MEMORY_REMAP_ADJUST()

/************************************
 *  HAL Level : Interrupt           *
 ************************************/
#define HW_ISR(hw)             HW##hw##_ISR
#define SET_HWISR(hw)          HW_ISR(hw)

/* ISR function name for interrupt controller platform */
#define FIQ_ISR                SET_HWISR(HW0_FIQ)
#define IRQ_ISR                SET_HWISR(HW1_IRQ)
#define SW0_ISR                SET_HWISR(HW6_SW0)

/* HAL for interrupt controller */
#define HAL_INTC_HWIRQ_ROUTE(_vector_, _hw_)                        \
do {                                                                \
        int n = _vector_ / 8, bits_shift = (_vector_ & 0x7 ) << 2;  \
        unsigned int val;                                           \
                                                                    \
        val = inw(INTC_BASE + IRQPRI_OFFSET(n)) & ~(0xf << bits_shift);\
        val |= ((_hw_ & 0xf) << bits_shift);                        \
        outw(INTC_BASE + IRQPRI_OFFSET(n), val);                    \
} while(0)

#define HAL_INTC_IRQ_CONFIGURE(_irqs_, _edge_, _falling_)           \
do {                                                                \
        if (_edge_) {                                               \
               outw(INTC_BASE + IRQTMR_OFFSET,                      \
                        inw(INTC_BASE + IRQTMR_OFFSET) | (_irqs_)); \
        } else {                                                    \
               outw(INTC_BASE + IRQTMR_OFFSET,                      \
                        inw(INTC_BASE + IRQTMR_OFFSET) & ~(_irqs_));\
        }                                                           \
        if (_falling_) {                                            \
                outw(INTC_BASE + IRQTLR_OFFSET,                     \
                        inw(INTC_BASE + IRQTLR_OFFSET) | (_irqs_)); \
        } else {                                                    \
                outw(INTC_BASE + IRQTLR_OFFSET,                     \
                        inw(INTC_BASE + IRQTLR_OFFSET) & ~(_irqs_));\
        }                                                           \
} while(0)

#define HAL_INTC_IRQ_MASK(_irqs_)                                   \
        outw(INTC_BASE + IRQER_OFFSET,                              \
                inw(INTC_BASE + IRQER_OFFSET) & ~(_irqs_))

#define HAL_INTC_IRQ_UNMASK(_irqs_)                                 \
        outw(INTC_BASE + IRQER_OFFSET,                              \
                inw(INTC_BASE + IRQER_OFFSET) | (_irqs_))

#define HAL_INTC_IRQ_CLEAR(_irqs_)                                  \
        outw(INTC_BASE + IRQICR_OFFSET, (_irqs_))

#define HAL_INTC_FIQ_CONFIGURE(_irqs_, _edge_, _falling_)           \
        HAL_INTC_IRQ_CONFIGURE(_irqs_, _edge_, _falling_)

#define HAL_INTC_FIQ_MASK(_irqs_)		HAL_INTC_IRQ_MASK(_irqs_)
#define HAL_INTC_FIQ_UNMASK(_irqs_)             HAL_INTC_IRQ_UNMASK(_irqs_)
#define HAL_INTC_FIQ_CLEAR(_irqs_)              HAL_INTC_IRQ_CLEAR(_irqs_)

/************************************
 *  HAL Level : Timer               *
 ************************************/
#define HAL_TIMER_INITIALIZE()                                      \
do {                                                                \
        outw(TM_CR, 0);                                             \
        outw(TM_MTCH1(0), 0);                                       \
        outw(TM_MTCH1(1), 0);                                       \
        outw(TM_MTCH1(2), 0);                                       \
        outw(TM_MSK, 0);                                            \
        outw(TM_STA, 0xFFFFFFFF);                                   \
        outw(TM_STA, 0);                                            \
} while(0)

#define HAL_TIMER_START(_tmr_)                                      \
        outw(TM_CR, inw(TM_CR) | (0x1 << (3 * (_tmr_))))

#define HAL_TIMER_STOP(_tmr_)                                       \
        outw(TM_CR, inw(TM_CR) & ~(0x1 << (3 * (_tmr_))))

#define HAL_TIMER_READ(_tmr_, _pvalue_)                             \
        *(volatile unsigned int *)_pvalue_ =                        \
                        inw(TM_LOAD(_tmr_)) - inw(TM_CNTR(_tmr_))

#define HAL_TIMER_SET_PERIOD(_tmr_, _period_)                       \
do {                                                                \
        outw(TM_LOAD(_tmr_), _period_);                             \
        outw(TM_CNTR(_tmr_), _period_);                             \
} while(0)

#define HAL_TIMER_IRQ_ENABLE(_tmr_, _ie_)                           \
do {                                                                \
        if(_ie_)                                                    \
                outw(TM_MSK, inw(TM_MSK) & ~(0x1 << (3 * (_tmr_))));\
        else                                                        \
                outw(TM_MSK, inw(TM_MSK) | (0x1 << (3 * (_tmr_)))); \
} while(0)

#define HAL_TIMER_IRQ_STATUS(_tmr_, _pvalue_)                       \
        *(volatile unsigned int *)_pvalue_ =                        \
                        (inw(TM_STA) & (0x7 << (3 * (_tmr_))))

#define HAL_TIMER_IRQ_CLR(_tmr_)                                    \
        outw(TM_STA, (0x7 << (3 * (_tmr_))))

/************************************
 *  HAL Level : GPIO                *
 ************************************/
#define HAL_GPIO_INITIALIZE(_pin_)                                  \
do {                                                                \
        outw(GPIO_BASE + GPIO_DIR_OFFSET,                           \
                inw(GPIO_BASE + GPIO_DIR_OFFSET) & ~_pin_);         \
        outw(GPIO_BASE + GPIO_IR_OFFSET,                            \
                inw(GPIO_BASE + GPIO_IR_OFFSET) | _pin_);           \
        outw(GPIO_BASE + GPIO_DCLEAR_OFFSET, -1);                   \
        outw(GPIO_BASE + GPIO_BPS_OFFSET, 0x0000FFFF);              \
        outw(GPIO_BASE + GPIO_BE_OFFSET, _pin_);                    \
        outw(GPIO_BASE + GPIO_IC_OFFSET, -1);                       \
        outw(GPIO_BASE + GPIO_IE_OFFSET, _pin_);                    \
} while(0)

#define HAL_GPIO_READ()         inw(GPIO_BASE + GPIO_DIN_OFFSET)

#define HAL_GPIO_IRQ_CLR(_pin_) outw(GPIO_BASE + GPIO_IC_OFFSET, _pin_)

#ifdef __ASSEMBLER__
/************************************
 *  HAL : AG102 defined vectors    *
 ************************************/
.macro  hal_hw_vectors
        vector Interrupt_HW0     //  (9) Interrupt HW0 (for IVB.IVIC_VER = 0, FIR)
        vector Interrupt_HW1     // (10) Interrupt HW1 (for IVB.IVIC_VER = 0, IRQ)
        vector Interrupt_UNDEF   // (11) Interrupt HW2
        vector Interrupt_UNDEF   // (12) Interrupt HW3
        vector Interrupt_UNDEF   // (13) Interrupt HW4
        vector Interrupt_UNDEF   // (14) Interrupt HW5
        vector Interrupt_HW6     // (15) Interrupt HW6 (for IVB.IVIC_VER = 0, SW0)
.endm

.macro  hal_hw_ISR
        Os_Trap_Interrupt_HW    HW0_FIQ
        Os_Trap_Interrupt_HW    HW1_IRQ
        Os_Trap_Interrupt_HW    HW6_SW0
.endm

.macro  hal_set_led, val
.endm
#endif  /* __ASSEMBLER__ */

#endif  /* AG102_REGS */
